package main

import (
	"os"

	log "github.com/rs/zerolog/log"
	"github.com/spf13/cobra"
	"github.com/spf13/viper"

	"git.tcp.direct/Mirrors/bitcask-mirror"
)

var mergeCmd = &cobra.Command{
	Use:     "merge",
	Aliases: []string{"clean", "compact", "defrag"},
	Short:   "Merges the Datafiles in the Database",
	Long: `This merges all non-active Datafiles in the Database and
compacts the data stored on disk. Old values are removed as well as deleted
keys.`,
	Args: cobra.ExactArgs(0),
	Run: func(cmd *cobra.Command, args []string) {
		path := viper.GetString("path")

		os.Exit(merge(path))
	},
}

func init() {
	RootCmd.AddCommand(mergeCmd)
}

func merge(path string) int {
	db, err := bitcask.Open(path)
	if err != nil {
		log.Error().Err(err).Msg("error opening database")
		return 1
	}

	if err = db.Merge(); err != nil {
		log.Error().Err(err).Msg("error merging database")
		return 1
	}

	return 0
}
